{{
Meta2 Compiler-Compiler for RamBlade

Copyright (c) 2010 Michael Green.  See end of file for terms of use.
}}


OBJ
   def     : "Definitions"             ' Definitions from DongleBasic
   ldrInit : "I2cSpiInit"              ' I2C/SPI driver init. & assembly portion
   mass    : "MassStorage"             ' EEPROM & SD card FAT file system support
   tv      : "OnePinTvInit"            ' 1-Pin TV Driver
   kbd     : "OnePinKBDInit"           ' 1-Pin Keyboard Driver

CON
   version   = 1                       ' Major version
   release   = 001                     ' Minor release
   testLevel = 5                       ' Test change level
   
   _clkmode  = def#clockMode
   _xinfreq  = def#xtalClock
   _free     = (def#endMemory - def#endFree + 3) / 4

   tvPin     = 22                      ' RamBlade console output pin
   kbdPin    = 23                      ' RamBlade console input pin

   linelen   = 63                      ' Maximum input line length
   tokens    =  4                      ' Maximum input tokens
   quote     = 34                      ' Double quote

   maxDirCol = 4                       ' Maximum directory columns
   maxListLn = 24                      ' Number of lines to list on screen
   
VAR
   byte kbdBuffer[linelen+1], fileLine[linelen+1]
   word tokPtrs[tokens+1]

PUB main | c, kbdPtr, tokPtr
  if long[def#initMarker] <> def#uniqueMark
    long[def#memPtr] := def#endFree                       ' Start at end of HUB memory
    long[def#randomSeed] := def#initMarker                ' Initialize random seed
    long[def#userPtr] := def#noSuchAddr
    tv.start(tvPin)
    tv.out(0)                                             ' Clear the screen
  tv.str(string("RamBlade Ouroboros"))
  if version > 0 or release > 0 or testLevel > 0
    tv.str(string(" Version "))                           ' Identify program
    tv.dec(version)                                       '  and version
    tv.out(".")
    if release < 100
      tv.out("0")
    if release < 10
      tv.out("0")
    tv.dec(release)
    if testLevel > 0
      tv.out("a"+testLevel-1)
  if long[def#initMarker] <> def#uniqueMark
    tv.str(string(def#Cr,"Type the <space> key ..."))
    long[def#kbdTiming] := c := kbd.calckbdtime(kbdpin)   ' Calculate the keyboard timing
    tv.str(string(def#Cr,"Keyboard timing = "))           ' Let driver calc the bittimes
    tv.dec(c & $FFFF)
    tv.out(",")
    tv.dec(c >> 16)
    tv.out(def#Cr)
    kbd.start(kbdpin, c & $FFFF, c >> 16)                 ' Start the 1pinKbd driver
    if not ldrInit.start                                  ' Now start up the I2C driver
      abort false
    long[def#initMarker] := def#uniqueMark                ' Basic initialization complete
  repeat
    tv.str(string(">>"))                                  ' Request a command
    kbdPtr~
    repeat                                                ' Input a line of text
      case c := kbd.in
        " ".."~":
          if kbdPtr < linelen                             ' If there's room in the buffer,
            kbdBuffer[kbdPtr++] := c                      '  save the character there
            tv.out(c)
        def#Bsp:
          if kbdPtr > 0                                   ' If not at left margin, backspace
            kbdPtr--
            tv.str(string(def#Bsp," ",def#Bsp))
        def#Cr:
          kbdBuffer[kbdPtr]~                              ' Mark end of input line
          kbdPtr~
          tv.out(def#Cr)
          quit
        other:
          next                                            ' Ignore other characters
    tokPtr~
    repeat while tokPtr < tokens
      repeat while kbdBuffer[kbdPtr++] == " "              ' Skip leading spaces
      tokPtrs[tokPtr++] := @kbdBuffer + --kbdPtr           ' Return address of token
      repeat
        if c := kbdBuffer[kbdPtr]                          ' Advance pointer if not end of line
          kbdPtr++
        if c == " " or c == ","                            ' Stop at next delimiter
          kbdBuffer[kbdPtr-1]~                             '  and mark end of string
          quit
        ifnot c                                            ' Stop at end of line
          tokPtrs[tokPtr]~
          tokPtr := tokens                                 ' Force end of tokens
          quit
    if tokPtrs[0] == 0
      tv.str(string("No command given",def#Cr))
    elseif strcomp(tokPtrs[0],string("run"))
      runCmd
    elseif strcomp(tokPtrs[0],string("list"))
      listCmd
    elseif strcomp(tokPtrs[0],string("dir"))
      dirCmd
    elseif strcomp(tokPtrs[0],string("erase"))
      delCmd
    elseif strcomp(tokPtrs[0],string("del"))
      delCmd
    elseif strcomp(tokPtrs[0],string("help"))
      helpCmd
    else
      tv.str(string("Unknown command ignored",def#Cr))

PUB runCmd
  if tokPtrs[1]
    mass.bootFile(tokPtrs[1])

PUB listCmd | c
  if \mass.mountSDVol(def#spiDO,def#spiClk,def#spiDI,def#spiCS) < 0
    tv.str(string("Can't mount SD card",def#Cr))
    return
  if \mass.openFile(tokPtrs[1],"r")
    tv.str(string("Can't open file",def#Cr))
    return
  repeat
    repeat maxListLn
      if result := fetchLine(@fileLine,linelen)
        quit
      tv.str(@fileLine)
      tv.out(def#Cr)
    if result
      quit
    tv.str(string("Press a key to continue "))
    c := kbd.in
    tv.out(def#Cr)
    if c == def#Bsp
      quit
  if \mass.closeFile < 0
    tv.str(string("Error closing file",def#Cr))
    return
  \mass.unmountSDVol

PUB fetchLine(buffer,length) | linePtr, c                 ' Fetch a line from
  linePtr~                                                '  the current file
  repeat
    result := (c := mass.getByte) < 0                     ' Get a byte from file
    if c <> def#Cr and c <> def#Lf and c > 0              ' Store character
      if linePtr < length                                 '  unless it's CR or
        byte[buffer][linePtr++] := c                      '   LF or EOF/Error
  until c == def#Cr or result
  byte[buffer][linePtr]~                                  ' Mark end of line

PUB dirCmd | anyFiles, column, name[4]
  if \mass.mountSDVol(def#spiDO,def#spiClk,def#spiDI,def#spiCS) < 0
    tv.str(string("Can't mount SD card",def#Cr))
    return
  mass.startSDFiles
  column := 0
  anyFiles := false
  repeat while mass.nextSDFiles(@name) == 0
    anyFiles := true
    tv.str(@name)
    if ++column < maxDirCol                               ' If not last column, fill
      repeat 13 - strsize(@name)                          '  rest of column with spaces
        tv.out(" ")
    else
      tv.out(def#Cr)
      column~
  if anyFiles
    tv.out(def#Cr)
  \mass.unmountSDVol

PUB delCmd | c
  tv.str(string("delete "))
  tv.str(tokPtrs[1])
  tv.str(string("? (y/N)"))
  repeat
    case c := kbd.in
      "Y", "y":
        quit
      "N", "n", def#Cr:
        tv.out(def#Cr)
        return
  tv.str(string(" ... "))
  if \mass.mountSDVol(def#spiDO,def#spiClk,def#spiDI,def#spiCS) < 0
    tv.str(string("can't mount SD card",def#Cr))
    return
  if \mass.openFile(tokPtrs[1],"d")
    tv.str(string("can't delete file",def#Cr))
    return
  if \mass.closeFile < 0
    tv.str(string("error deleting file",def#Cr))
    return
  \mass.unmountSDVol
  tv.str(string("ok",def#Cr))

PUB helpCmd
  tv.str(string("Built-in Commands:",def#Cr))
  tv.str(string(" run <file>",def#Cr))
  tv.str(string(" dir",def#Cr))
  tv.str(string(" del <file>",def#Cr))
  tv.str(string(" list <file>",def#Cr))

PUB readSRAM(Addr)
  outa := Addr & $7FFFF
  dira |= $0007_FFFF          'address outputs                (+CE +WE =1 =deselect)
  dira |= $0010_0000          'address & CE outputs           (-CE     =0 =read)
  result := ina >> 24         'read from sram
  dira &= $FFCF_FFFF          'data & address outputs         (+CE +WE =1 =deselect)
  dira &= $00C8_0000          'allow for other I/O pins in use (19, 22, 23)

PUB writeSRAM(Addr,Data)
  outa := (Data << 24) | Addr 'data + address + -CE
  dira |= $FF07_FFFF          'data & address outputs         (+CE +WE =1 =deselect)
  dira |= $0030_0000          'data, address, CE & WE outputs (-CE -WE =0 =write)
  dira &= $FFCF_FFFF          'data & address outputs         (+CE +WE =1 =deselect)
  dira &= $00C8_0000          'allow for other I/O pins in use (19, 22, 23)

{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
